#define G_LOG_DOMAIN "gpod-feed"

#include "config.h"

#include "gpod-feed.h"

#include <glib.h>
#include <glib-object.h>
#include <glib/gi18n.h>

/**
 * SECTION:gpod-feed
 * @title: GpodFeed
 * @short_description: A #GtkBox widget for list feeds
 * @include: gpod-feed.h
 * @see_also: #GpodFeed, #GpodFeed
 *
 * The #GpodFeed is an instance #GtkBox, and acts as the primary
 * window/widget for listing feeds in the #GpodWindow<!-- -->'s #GtkStack.
 **/

typedef struct _GpodFeedPrivate
{
    gchar *title;
    gchar *description;
    gchar *link;
    gchar *publish_date;
    gchar *last_build_date;
    gchar *image;

    GList *items;
} GpodFeedPrivate;

struct _GpodFeed
{
    GObject parent;
};

G_DEFINE_TYPE_WITH_PRIVATE (GpodFeed, gpod_feed, G_TYPE_OBJECT)

enum
{
    PROP_0,
    PROP_TITLE,
    PROP_DESCRIPTION,
    PROP_LINK,
    PROP_PUBLISH_DATE,
    PROP_LAST_BUILD_DATE,
    PROP_IMAGE,
    NUM_PROPERTIES
};

static GParamSpec *properties[NUM_PROPERTIES] = { NULL, };

static void
gpod_feed_set_property (GObject      *object,
                        guint         property_id,
                        const GValue *value,
                        GParamSpec   *pspec)
{
    GpodFeed *self = GPOD_FEED (object);
    GpodFeedPrivate *priv = gpod_feed_get_instance_private (self);

    switch (property_id)
    {
        case PROP_TITLE:
            priv->title = g_value_dup_string (value);
            break;

        case PROP_DESCRIPTION:
            priv->description = g_value_dup_string (value);
            break;

        case PROP_LINK:
            priv->link = g_value_dup_string (value);
            break;

        case PROP_PUBLISH_DATE:
            priv->publish_date = g_value_dup_string (value);
            break;

        case PROP_LAST_BUILD_DATE:
            priv->last_build_date = g_value_dup_string (value);
            break;

        case PROP_IMAGE:
            priv->image = g_value_dup_string (value);
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
            break;
    }
}

static void
gpod_feed_get_property (GObject      *object,
                        guint         property_id,
                        GValue       *value,
                        GParamSpec   *pspec)
{
    GpodFeed *self = GPOD_FEED (object);
    GpodFeedPrivate *priv = gpod_feed_get_instance_private (self);

    switch (property_id)
    {
        case PROP_TITLE:
            g_value_set_string (value, priv->title);
            break;

        case PROP_DESCRIPTION:
            g_value_set_string (value, priv->description);
            break;

        case PROP_LINK:
            g_value_set_string (value, priv->link);
            break;

        case PROP_PUBLISH_DATE:
            g_value_set_string (value, priv->publish_date);
            break;

        case PROP_LAST_BUILD_DATE:
            g_value_set_string (value, priv->last_build_date);
            break;

        case PROP_IMAGE:
            g_value_set_string (value, priv->image);
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
            break;
    }
}

/*
 * gpod_feed_finalize:
 * @object: An instance of #GpodFeed
 *
 * Does any last minute clean-ups when the class ends.
 */
static void
gpod_feed_finalize (GObject *object)
{
    G_OBJECT_CLASS (gpod_feed_parent_class)->finalize(object);
}

/*
 * gpod_feed_init:
 * @self: An instance of #GpodFeed
 *
 * Initializes various aspects of the #GpodFeed. Anything the #GpodFeed
 * would need before becoming presented should be done here.
 */
static void
gpod_feed_init (GpodFeed *self)
{

}

/*
 * gpod_feed_class_init:
 * @klass: An instance of #GpodFeedClass
 *
 * Initializes the various virtural functions in #GpodFeedClass. In
 * addtition it connects the various #GtkWidgets from the gresource file using
 * gtk_widget_class_set_template_from_resource().
 */

static void
gpod_feed_class_init (GpodFeedClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->set_property = gpod_feed_set_property;
    object_class->get_property = gpod_feed_get_property;
    object_class->finalize = gpod_feed_finalize;

    properties[PROP_TITLE] =
        g_param_spec_string ("title",
                             "Title",
                             "Feed title.",
                             NULL,
                             G_PARAM_READWRITE);

    properties[PROP_DESCRIPTION] =
        g_param_spec_string ("description",
                             "Description",
                             "Description of the feed.",
                             NULL,
                             G_PARAM_READWRITE);

    properties[PROP_LINK] =
        g_param_spec_string ("link",
                             "Link",
                             "Link to the feed site.",
                             NULL,
                             G_PARAM_READWRITE);

    properties[PROP_PUBLISH_DATE] =
        g_param_spec_string ("publish-date",
                             "Publish Date",
                             "Publish date of the feed.",
                             NULL,
                             G_PARAM_READWRITE);

    properties[PROP_LAST_BUILD_DATE] =
        g_param_spec_string ("last-build-date",
                             "Last Build Date",
                             "Date the feed was last built.",
                             NULL,
                             G_PARAM_READWRITE);

    properties[PROP_IMAGE] =
        g_param_spec_string ("image",
                             "Image",
                             "Feed image.",
                             NULL,
                             G_PARAM_READWRITE);

    g_object_class_install_properties (object_class,
                                       NUM_PROPERTIES,
                                       properties);
}

/*
 * *****************************************************************************
 * API
 * *****************************************************************************
 */
/**
 * gpod_feed_new:
 *
 * Creates a new #GpodFeed, a custom widget to show all feeds user is
 * subscribed to.
 *
 * ## Usage
 * |[
 * GpodFeed *feed;
 *
 * feed = gpod_feed_new ();
 * ]|
 *
 * Returns: #GpodFeed
 */
GpodFeed *
gpod_feed_new ()
{
    return g_object_new (GPOD_TYPE_FEED,
                         NULL);
}

void
gpod_feed_add_item (GpodFeed     *self,
                    GpodFeedItem *feed_item)
{

}

/**
 * gpod_feed_get_title:
 * @self: an #GpodFeed
 *
 * Gets the title of @self.
 *
 * Returns: (transfer full) (nullable): the feed title of @self, free with
 *          g_free()
 */
gchar *
gpod_feed_get_title (GpodFeed *self)
{
    GValue val = G_VALUE_INIT;
    gchar *retval;

    g_assert (GPOD_IS_FEED (self));

    g_value_init (&val, G_TYPE_STRING);

    g_object_get_property (G_OBJECT (self), "title", &val);

    retval = g_value_dup_string (&val);

    g_value_unset (&val);

    return retval;
}
